/**
 * Copyright 2012 the project-owners
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package inti.ws.spring.resource.config;

import inti.ws.spring.resource.WebResource;

import java.io.File;
import java.io.FilenameFilter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.inject.Inject;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import org.apache.http.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

@Component
public class ResourceConfigProviderImpl implements InitializingBean, ResourceConfigProvider {
    
    private static final String DEFAULT_CONFIG_NAME = "_default";
    private static final String CONFIG_EXTENSION = ".json";
    private static final String HOSTS_DIR = "/WEB-INF/hosts/";
    private static final Logger LOGGER = LoggerFactory.getLogger(ResourceConfigProviderImpl.class);
    
    @Inject
    ServletContext ctx;
    @Inject
    ConfigParserProvider configParserProvider;
    boolean updateConfig = false;
    Map<String, Long> lastModifies = new HashMap<String, Long>();
    Map<String, Map<String, ResourceConfig>> configs = new HashMap<>();
    ObjectMapper mapper = new ObjectMapper();
    FilenameFilter filter = new FilenameFilter() {
        
        @Override
        public boolean accept(File dir, String name) {
            return name != null && name.endsWith(CONFIG_EXTENSION);
        }
        
    };
    
    @Override
    public void afterPropertiesSet() throws Exception {
        updateConfig = true;
        Set<String> paths = ctx.getResourcePaths(HOSTS_DIR);
        
        for (String path : paths) {
            if (path.endsWith(CONFIG_EXTENSION)) {
                LOGGER.debug("afterPropertiesSet - scanning config-file={}", path);
                update(path.substring(path.indexOf(HOSTS_DIR) + HOSTS_DIR.length(), path.lastIndexOf(CONFIG_EXTENSION)));
            }
        }
        updateConfig = !(System.getProperty("DEBUG_MODE") == null || !System.getProperty("DEBUG_MODE").equals("true"));
    }
    
    @Override
    public Map<String, ResourceConfig> getResourceConfig(HttpServletRequest request) {
        String key = getKey(request);
        
        if (updateConfig) {
            try {
                update(key);
            } catch (Exception exception) {
                LOGGER.error("", exception);
            }
        }
        
        return configs.get(key);
    }
    
    protected String getKey(HttpServletRequest request) {
        String key = request.getHeader(HttpHeaders.HOST);
        int index;
        
        if (key == null) {
            key = DEFAULT_CONFIG_NAME;
        } else if ((index = key.indexOf(":")) != -1) {
            key = key.substring(0, index);
            if (!configs.containsKey(key)) {
                key = DEFAULT_CONFIG_NAME;
            }
        }
        
        return key;
    }
    
    @SuppressWarnings({ "unchecked", "rawtypes" })
    protected void update(String key) throws Exception {
        File configFile = new File(new File(ctx.getResource(HOSTS_DIR).toURI()), key + CONFIG_EXTENSION);
        Map<String, ConfigParser<? extends WebResource>> parsers;
        ConfigParser<? extends WebResource> parser;
        JsonNode configNode;
        Entry<String, JsonNode> field;
        Iterator<Entry<String, JsonNode>> fields;
        Map<String, ? extends WebResource> resources;
        Map<String, ResourceConfig> resourceConfigs = new HashMap<>();
        Map<String, Map<String, ? extends WebResource>> resourceStack = new HashMap<>();
        ResourceConfig config;
        
        if (lastModifies.containsKey(key) && configFile.lastModified() <= lastModifies.get(key)) {
            return;
        } else {
            lastModifies.put(key, configFile.lastModified());
        }
        
        configNode = mapper.readTree(configFile);
        fields = configNode.fields();
        parsers = configParserProvider.getConfigParsers(getClass().getClassLoader());
        
        LOGGER.debug("update - scanning with parsers={}", parsers);
        
        while (fields.hasNext()) {
            field = fields.next();
            parser = parsers.get(field.getKey());
            LOGGER.debug("update - got parser for field={}: {}", field.getKey(), parser);
            config = new ResourceConfig();
            if (parser != null) {
                resources = parser.instanceWebResources(mapper, field.getValue());
                resourceStack.put(field.getKey(), resources);
                
                config.setResources(resources);
                config.setSettings(parser.getConfigParserSettings());
                if (parser.getConfigParserSettings().getRoutes() != null) {
                    for (String route : parser.getConfigParserSettings().getRoutes()) {
                        resourceConfigs.put(route, config);
                    }
                }
            }
        }
        
        fields = configNode.fields();
        while (fields.hasNext()) {
            field = fields.next();
            parser = parsers.get(field.getKey());
            if (parser != null) {
                parser.configureWebResources(mapper, field.getValue(), (Map) resourceStack.get(field.getKey()),
                        (Map) resourceStack);
            }
        }
        
        LOGGER.debug("update - added config with key={}: {}", key, resourceConfigs);
        
        configs.put(key, resourceConfigs);
    }
    
}
